SystemOrganization addCategory: #StartupPreferences!

!SettingBrowser methodsFor: '*StartupPreferences' stamp: 'BenjaminVanRyseghem 8/1/2012 15:28'!
exportSettings
	| title nodes actions index groupSize |
	
	groupSize := 50.
	index := 1.
	nodes := self treeHolder nodeList.
	title := 'Exporting settings'.
	title
		displayProgressFrom: 1 
		to: nodes size 
		during: [:bar | actions := nodes collectWithIndex: [:e :i | 
											bar current: i.
											bar label: (String streamContents: [:s | 
															s << title << ' (' << (e item label) << ')']).
											e item exportSettingAction ]].
		
		actions := actions reject: [:e | e isNil ].
		
	actions groupsOf: groupSize atATimeDo: [:l | 
		StartupLoader default addAtStartupInPreferenceVersionFolder: l named: 'exported_settings',index printString, '.st'.
		index := index +1 ].
	StartupLoader default addAtStartupInPreferenceVersionFolder: (actions last: (actions size rem: groupSize)) named: 'exported_settings',index printString, '.st'! !

!PragmaSetting methodsFor: '*StartupPreferences' stamp: 'BenjaminVanRyseghem 8/1/2012 15:44'!
exportSettingAction
	
	^ nil! !

Object subclass: #PreferencesHandler
	instanceVariableNames: 'firstLink actions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StartupPreferences'!

!PreferencesHandler commentStamp: '<historical>' prior: 0!
I am the one which manage the links of the chain of responsibilites to retreive the good preference file!

!PreferencesHandler methodsFor: 'actions' stamp: 'SeanDeNigris 8/4/2012 10:48'!
buildActionList
	"All found files will be executed."
	
	| directories |
	directories := OrderedCollection new.
	directories add: [ self lookInGeneralPreferencesFolder ].
	directories add: [ self lookInPreferencesVersionFolder ].
	directories add: [ self lookInImageFolder ].
	^ directories! !

!PreferencesHandler methodsFor: 'initialization' stamp: 'SeanDeNigris 8/4/2012 10:48'!
initialize

	super initialize.
	actions := self buildActionList.! !

!PreferencesHandler methodsFor: 'private' stamp: 'SeanDeNigris 7/8/2012 11:56'!
lookInFolder: folder

	| result |

	self flag: 'I should be #lookInFolder:forFilesMatching:, so that #lookInImageFolder can delegate to me, like the other two lookup methods'.
	result := OrderedCollection new.
	folder exists ifFalse: [ ^ nil ].
	
	"To filter resources files such as .DS_Store"
	folder fileNames do: [:name |
		name first = $.
			ifFalse: [ result add: (folder / name)]].

	^ result isEmpty
		ifTrue: [ nil ]
		ifFalse: [ result ]! !

!PreferencesHandler methodsFor: 'private' stamp: 'SeanDeNigris 8/4/2012 10:56'!
lookInFolder: folder forFilesMatching: pattern

	| result files |
	result := OrderedCollection new.
	folder exists ifFalse: [ ^ nil ].
	
	files := folder filesMatching: pattern.
	files do: [ :e | result add: e ].

	^ result isEmpty
		ifTrue: [ nil ]
		ifFalse: [ result ]! !

!PreferencesHandler methodsFor: 'actions' stamp: 'SeanDeNigris 8/4/2012 10:53'!
lookInGeneralPreferencesFolder

	^ self lookInFolder: StartupLoader preferencesGeneralFolder forFilesMatching: '*.st'.! !

!PreferencesHandler methodsFor: 'actions' stamp: 'SeanDeNigris 8/4/2012 11:26'!
lookInImageFolder

	^ self lookInFolder: FileSystem workingDirectory forFilesMatching: StartupLoader startupScriptName.! !

!PreferencesHandler methodsFor: 'actions' stamp: 'SeanDeNigris 8/4/2012 10:53'!
lookInPreferencesVersionFolder

	^ self lookInFolder: StartupLoader preferencesVersionFolder forFilesMatching: '*.st'.! !

!PreferencesHandler methodsFor: 'actions' stamp: 'SeanDeNigris 8/4/2012 10:53'!
perform

	| results |
	results := OrderedCollection new.
	actions do: [:each |
		each value ifNotNil: [ :res |  results addAll: res ]
	].
	^ results
! !

Object subclass: #StartupAction
	instanceVariableNames: 'code hasBeenExecuted runOnce nameOfItem name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StartupPreferences'!

!StartupAction commentStamp: '<historical>' prior: 0!
I am an atomic item used to execute a portion of code at image startup.!

!StartupAction class methodsFor: 'instance creation' stamp: 'IgorStasenko 2/4/2012 19:09'!
name: nameOfItem code: code

	^ self new
		code: code;
		name: nameOfItem;
		yourself! !

!StartupAction class methodsFor: 'instance creation' stamp: 'IgorStasenko 2/4/2012 19:09'!
name: nameOfItem code: code runOnce: aBoolean

	^ self new
		code: code;
		name: nameOfItem;
		runOnce: aBoolean;
		yourself! !

!StartupAction methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 19:07'!
= anObject

	anObject class = self class ifFalse: [ ^ false ].
	^ self code = anObject code and: [ self name = anObject name and: [ self runOnce = anObject runOnce ]]! !

!StartupAction methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/1/2011 10:06'!
code

	^ code! !

!StartupAction methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/1/2011 10:05'!
code: anObject
	
	code := anObject! !

!StartupAction methodsFor: 'execution' stamp: 'MarianoMartinezPeck 5/3/2012 23:11'!
execute

	| block correctlyExecuted |
	(self hasBeenExecuted and: [ self runOnce ]) ifTrue: [ ^ self ].
	block := self code isBlock
				ifTrue: [ self code ]
				ifFalse: [[ self class compilerClass evaluate: self code ]].
	correctlyExecuted := true. 
	block on: Halt,Error do: [:ex | StartupLoader default errors add: {ex. self}. correctlyExecuted := false].
	self hasBeenExecuted: correctlyExecuted.! !

!StartupAction methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/1/2011 10:06'!
hasBeenExecuted

	^ hasBeenExecuted ifNil: [ hasBeenExecuted := false ]! !

!StartupAction methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/1/2011 10:05'!
hasBeenExecuted: anObject
	
	hasBeenExecuted := anObject! !

!StartupAction methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 19:07'!
name

	^ name! !

!StartupAction methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 19:07'!
name: anObject
	
	name := anObject! !

!StartupAction methodsFor: 'printing' stamp: 'IgorStasenko 2/4/2012 19:07'!
printOn: aStream

	aStream <<'a ' << self class name << '(' << self name << ')'! !

!StartupAction methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 18:10'!
runOnce

	^ runOnce ifNil: [ runOnce := false ]! !

!StartupAction methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 18:11'!
runOnce: anObject
	
	runOnce := anObject! !

!StartupAction methodsFor: 'printing' stamp: 'BenjaminVanRyseghem 8/1/2012 14:29'!
storeOn: aStream

	aStream << self class name << ' name: '<< self name printString <<' code: '.
	self code storeOn: aStream.
	self runOnce ifTrue: [ aStream << ' runOnce: true']! !

Object subclass: #StartupLoader
	instanceVariableNames: 'errors actions'
	classVariableNames: 'AllowStartupScript UniqueInstance'
	poolDictionaries: ''
	category: 'StartupPreferences'!

!StartupLoader commentStamp: '<historical>' prior: 0!
StartupLoader searches for and executes .st files from certain locations.  To find these it searches for a '.config' folder in the folder next up from where the image file sits.  Then it looks in the next folder up again and so on until reaching the root folder.  When a '.config' folder is found, StartupLoader looks within this for a 'pharo' folder. This contains the startup scripts common to all versions of Pharo, and also optionally a folder per Pharo version holding startup scripts suitable for that version only.  So a typical directory layout might be...

.../some/folders/pharo/Content/Resources/pharo.image.
.../some/folders/pharo/Content/Resources/startup.st
.../some/folders/.config/pharo/author.st
.../some/folders/.config/pharo/useSharedCache.st
.../some/folders/.config/pharo/1.4/mystartupFor14only.st
.../some/folders/.config/pharo/2.0/mystartupFor20only.st

(**Note however that '.config' is an invalid filename on Windows, so '..config' is used instead)

IMPORTANT: StartupLoader will search for a folder '.config' starting from the image directory until the root of the filesystem. What happens if no folder is found? It creates '.config' in the image folder. However, it is recommended that you create the '.config' following the standard, that is, in the $HOME.

To know the real values for you...
Print the result of "FileDirectory preferencesGeneralFolder" which holds the startup scripts common to all versions of Pharo.
Print the result of "FileDirectory preferencesVersionFolder" which holds the startup scripts specific to the version of the current image.

-----------


StartupLoader example

will define a script sample startup.st in your unix root on unix 

Its contents is 

StartupLoader default executeAtomicItems: {
	StartupAtomicItem name: 'Open Help' code: 'Workspace openContents: ''Here is just an example of how to use the StartupLoader.
I should only be displayed once.
	
You can also see StartupLoader class>>#example'' label: ''Help''' isSingleton: true.
	StartupAtomicItem name: 'Open Workspace' code: 'Workspace openContents: ''I should be displayed each time'''.
}

For a more complete example, see StartupLoader class>>#example2!

!StartupLoader class methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 19:17'!
allowStartupScript

	^ AllowStartupScript ifNil: [ AllowStartupScript := true ]! !

!StartupLoader class methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 19:17'!
allowStartupScript: aBoolean

	AllowStartupScript := aBoolean! !

!StartupLoader class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 11/1/2011 08:56'!
default

	^ UniqueInstance ifNil: [ UniqueInstance := super basicNew initialize ]! !

!StartupLoader class methodsFor: 'example' stamp: 'MarianoMartinezPeck 5/6/2012 19:32'!
example
	" an example of script generation "
	| item1 item2 |
	item1 := StartupAction name: 'Open Help' code: 'Workspace openContents: ''Here is just an example of how to use the StartupLoader.
I should only be displayed once.
	
 You can also see StartupLoader class>>#example'' label: ''Help''' runOnce: true.
	item2 := StartupAction name: 'Open Workspace' code:  [ Workspace openContents: 'I should be displayed each time' ].
	StartupLoader default addAtStartupInGeneralPreferenceFolder: {item1. item2}.
	
	StartupLoader default loadStartupScript.
	StartupLoader default loadStartupScript.! !

!StartupLoader class methodsFor: 'example' stamp: 'MarianoMartinezPeck 5/9/2012 19:05'!
example2
	" it's my personal script provided as example"
	"self example2"
	| items |
	items := OrderedCollection new.
	items add: (StartupAction name: 'Set the Author' code: [ Author fullName: Author fullName printString ]). "replace it by your name"
	
	StartupLoader default addAtStartupInPreferenceVersionFolder: items named: 'author.st'.
	items removeAll.
	
	items add: (StartupAction name: 'Debugger option' code: [ Debugger alwaysOpenFullDebugger: true ]).
	items add: (StartupAction name: 'Dragging Option' code: [ UITheme defaultSettings fastDragging: true ]).
	items add:  (StartupAction name: 'CMD Left Shortcut' code: [ TasklistMorph useHistoricalShortcuts: false ]).
	items add: (StartupAction name: 'Dialog Auto Accept' code: [ TextEditorDialogWindow autoAccept: true ]).
	StartupLoader default addAtStartupInPreferenceVersionFolder: items named: 'settings.st'.
	items removeAll.
	
	items add: (StartupAction name: 'Fonts option' code: [ FreeTypeSystemSettings loadFt2Library: true.
FreeTypeFontProvider current updateFromSystem.
StandardFonts defaultFont: (LogicalFont familyName: 'Lucida Grande' pointSize: 10) forceNotBold.
GraphicFontSettings resetAllFontToDefault.
StandardFonts codeFont: (LogicalFont familyName: 'Consolas' pointSize: 10).] runOnce: true).
	
	StartupLoader default addAtStartupInPreferenceVersionFolder: items named: 'fonts.st'.
	items removeAll.
	StartupLoader default addAtStartupInImageDirectory: items.
	
	StartupLoader default loadStartupScript.! !

!StartupLoader class methodsFor: 'example' stamp: 'MarianoMartinezPeck 5/9/2012 19:05'!
example3
	" it's my personal script provided as example"
	"self example3"
	| items |
	items := OrderedCollection new.
	items add: (StartupAction name: 'Self halt' code: [ self halt ]). "replace it by your name"
	items add: (StartupAction name: 'WS' code: [ Workspace open ]). "replace it by your name"
	StartupLoader default addAtStartupInPreferenceVersionFolder: items named: 'author.st'.

	StartupLoader default loadStartupScript.! !

!StartupLoader class methodsFor: 'example' stamp: 'MarianoMartinezPeck 5/9/2012 19:05'!
example4
	"self example4"
	| items |
	items := OrderedCollection new.
	
	items add: (StartupAction name: 'General Preferences for all Pharo versions' code: [ 
		FileStream stdout lf; nextPutAll: 'Setting general preferences for all Pharo versions'; lf.
		FileStream stdout lf; nextPutAll: 'Finished'; lf.
		 ]).
	StartupLoader default addAtStartupInGeneralPreferenceFolder: items named: 'generalSettings.st'.
	
	items removeAll.
	items add: (StartupAction name: 'Settings' code: [ 
		FileStream stdout lf; nextPutAll: 'Setting general preferences for Pharo 2.0'; lf.
		FileStream stdout lf; nextPutAll: 'Finished'; lf.
		 ]).
	StartupLoader default addAtStartupInPreferenceVersionFolder: items named: 'settings.st'.
	
	items removeAll.
	items add: (StartupAction name: 'Image diretory' code: [ 
		FileStream stdout lf; nextPutAll: 'Setting preferences for image directory'; lf.
		FileStream stdout lf; nextPutAll: 'Finished'; lf.
		 ]).
	StartupLoader default addAtStartupInImageDirectory: items ! !

!StartupLoader class methodsFor: 'initialize' stamp: 'BenjaminVanRyseghem 11/1/2011 10:39'!
initialize

	super initialize.
	Smalltalk addToStartUpList: self.! !

!StartupLoader class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 11/1/2011 08:56'!
new

	^ self error: 'use #default instead'! !

!StartupLoader class methodsFor: 'private' stamp: 'SeanDeNigris 7/12/2012 10:14'!
preferencesGeneralFolder

	^ FileLocator preferences asFileReference / 'pharo'.! !

!StartupLoader class methodsFor: 'private' stamp: 'SeanDeNigris 7/12/2012 10:14'!
preferencesVersionFolder

	^ self preferencesGeneralFolder asFileReference / SystemVersion current dottedMajorMinor! !

!StartupLoader class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 11/1/2011 09:14'!
reset

	UniqueInstance := nil! !

!StartupLoader class methodsFor: 'initialize' stamp: 'BenjaminVanRyseghem 2/10/2012 18:44'!
startUp: resuming

	(self allowStartupScript and: [ resuming ])
		ifTrue: [ Smalltalk addDeferredStartupAction: [ self default loadStartupScript ]]! !

!StartupLoader class methodsFor: 'private' stamp: 'MarianoMartinezPeck 5/7/2012 18:59'!
startupGeneralPreferencesFolderScriptName

	^ 'startupPharo.st'! !

!StartupLoader class methodsFor: 'menu' stamp: 'MarcusDenker 3/24/2012 17:07'!
startupLoaderMenuOn: aBuilder 
	<worldMenu>
	
	(aBuilder item: #startupLoader)
		action: [ self default loadStartupScript ];
		label: 'Run startup scripts';
		parent: #System;
		order: 2;
		help: 'Run startup "scripts';
		icon: UITheme current smallJustifiedIcon! !

!StartupLoader class methodsFor: 'setting' stamp: 'IgorStasenko 2/4/2012 19:17'!
startupPreferencesSettingsOn: aBuilder
	<systemsettings>
	
	(aBuilder group: #'Startup Preferences')
		target: self;
		label: 'Startup Preferences' translated;
		description: 'Settings related to Startup Preferences' translated;
		with: [
			(aBuilder setting: #allowStartupScript)
				label: 'Allow the preferences to be loaded at startup' translated;
				description: 'If true, the preferences will be loaded' translated ]! !

!StartupLoader class methodsFor: 'private' stamp: 'MarianoMartinezPeck 5/9/2012 19:05'!
startupPreferencesVersionFolderScriptName

	^ 'startupPharo',SystemVersion current dottedMajorMinor,'.st'! !

!StartupLoader class methodsFor: 'private' stamp: 'MarianoMartinezPeck 5/7/2012 18:59'!
startupScriptName

	^ 'startup.st'! !

!StartupLoader methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 19:06'!
actions

	^ actions! !

!StartupLoader methodsFor: 'accessing' stamp: 'IgorStasenko 2/4/2012 19:08'!
add: anAction

	^ actions at: anAction name ifAbsentPut: [ anAction ]! !

!StartupLoader methodsFor: 'private' stamp: 'SeanDeNigris 7/8/2012 12:05'!
addAtStartup: aCollection inDirectory: aFileReference named: fileName

	| scriptFile |
	aFileReference ensureDirectory.
	scriptFile := aFileReference / fileName.
	FileStream forceNewFileNamed: scriptFile fullName do: [ :stream | stream nextPutAll: (self buildStreamFor: aCollection) ].
! !

!StartupLoader methodsFor: 'script generation' stamp: 'CamilloBruni 5/26/2012 12:31'!
addAtStartupInGeneralPreferenceFolder: aCollection 
	self 
		addAtStartup: aCollection 
		inDirectory: self preferencesGeneralFolder 
		named: self class startupGeneralPreferencesFolderScriptName! !

!StartupLoader methodsFor: 'script generation' stamp: 'CamilloBruni 5/26/2012 12:31'!
addAtStartupInGeneralPreferenceFolder: aCollection named: fileName
	self 
		addAtStartup: aCollection 
		inDirectory: self preferencesGeneralFolder 
		named: fileName! !

!StartupLoader methodsFor: 'script generation' stamp: 'CamilloBruni 5/26/2012 12:32'!
addAtStartupInImageDirectory:  aCollection
	self 
		addAtStartup: aCollection 
		inDirectory: FileSystem workingDirectory 
		named: self class startupScriptName
! !

!StartupLoader methodsFor: 'script generation' stamp: 'CamilloBruni 5/26/2012 12:32'!
addAtStartupInPreferenceVersionFolder: aCollection 
	self 
		addAtStartup: aCollection 
		inDirectory: self preferencesVersionFolder 
		named: self class startupPreferencesVersionFolderScriptName! !

!StartupLoader methodsFor: 'script generation' stamp: 'CamilloBruni 5/26/2012 12:33'!
addAtStartupInPreferenceVersionFolder: aCollection named: fileName
	self 
		addAtStartup: aCollection 
		inDirectory: self preferencesVersionFolder 
		named: fileName! !

!StartupLoader methodsFor: 'private' stamp: 'BenjaminVanRyseghem 12/14/2011 13:42'!
buildStreamFor: aCollection
	| stream |
	
	stream := WriteStream on: ''.

	stream
		nextPutAll: 'StartupLoader default executeAtomicItems: {';
		nextPutAll: String crlf.
	aCollection do: [:item |
		stream 
			nextPut: Character tab;
			nextPutAll: item storeString;
			nextPut: $.;
			nextPutAll: String crlf ].
	stream
		nextPut: $};
		nextPut: $.;
		nextPutAll: String crlf;
		nextPutAll: String crlf.
	^ stream contents! !

!StartupLoader methodsFor: 'cleaning' stamp: 'MarianoMartinezPeck 5/7/2012 18:14'!
cleanSavedActionsAndErrors
	actions := Dictionary new. 
	errors := OrderedCollection new. ! !

!StartupLoader methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 11/28/2011 17:45'!
errors

	^ errors! !

!StartupLoader methodsFor: 'code execution' stamp: 'SeanDeNigris 6/21/2012 09:13'!
executeAtomicItems: aCollection
	"No sender but send from the generated script"

	'Load Startup Preferences'
		displayProgressFrom: 0 to: aCollection size
		during: [:bar | 
			aCollection doWithIndex: [:item :index | 
				bar current: index.
				(self add: item) execute ]]! !

!StartupLoader methodsFor: 'initialize' stamp: 'IgorStasenko 2/4/2012 19:11'!
initialize

	super initialize.
	actions := Dictionary new. 
	errors := OrderedCollection new. ! !

!StartupLoader methodsFor: 'startup loading' stamp: 'BenjaminVanRyseghem 12/13/2011 11:29'!
loadPreferences

	self retreiveFilesStream do: [:file | file fileIn ].! !

!StartupLoader methodsFor: 'startup loading' stamp: 'SeanDeNigris 8/8/2012 12:30'!
loadStartupScript

	errors removeAll.
	self loadPreferences.
	errors ifNotEmpty: [ :errs | errs explore ]. ! !

!StartupLoader methodsFor: 'utility' stamp: 'SeanDeNigris 7/8/2012 11:49'!
preferencesGeneralFolder

	^ self class preferencesGeneralFolder.! !

!StartupLoader methodsFor: 'utility' stamp: 'SeanDeNigris 7/8/2012 11:50'!
preferencesVersionFolder

	^ self class preferencesVersionFolder.! !

!StartupLoader methodsFor: 'cleaning' stamp: 'MarianoMartinezPeck 5/7/2012 18:15'!
removeAllScriptsAndCleanSavedActions
	self removeAllScriptsFromAllDirectories.
	self cleanSavedActionsAndErrors.
	! !

!StartupLoader methodsFor: 'cleaning' stamp: 'MarianoMartinezPeck 5/9/2012 19:28'!
removeAllScriptsFromAllDirectories
	self removeScriptFromImageDirectory.
	self removeAllScriptsFromPreferencesVersionDirectory.
	self removeAllScriptsFromGeneralPreferencesDirectory.
! !

!StartupLoader methodsFor: 'cleaning' stamp: 'StephaneDucasse 6/17/2012 18:28'!
removeAllScriptsFromDirectory: aDirectory
	aDirectory deleteAllChildren! !

!StartupLoader methodsFor: 'cleaning' stamp: 'CamilloBruni 5/26/2012 12:33'!
removeAllScriptsFromGeneralPreferencesDirectory
	self removeAllScriptsFromDirectory: self preferencesGeneralFolder.
	
	
! !

!StartupLoader methodsFor: 'cleaning' stamp: 'CamilloBruni 5/26/2012 12:33'!
removeAllScriptsFromPreferencesVersionDirectory
	self removeAllScriptsFromDirectory: self preferencesVersionFolder.

! !

!StartupLoader methodsFor: 'cleaning' stamp: 'CamilloBruni 5/26/2012 12:33'!
removeScriptFromImageDirectory
	(FileSystem workingDirectory / self class startupScriptName) delete.
! !

!StartupLoader methodsFor: 'startup loading' stamp: 'BenjaminVanRyseghem 12/13/2011 11:29'!
retreiveFilesStream

	^ PreferencesHandler new perform! !

!SettingDeclaration methodsFor: '*StartupPreferences' stamp: 'BenjaminVanRyseghem 8/1/2012 14:52'!
exportSettingAction
	| targetSymbol currentValue |
	
	targetSymbol := self target isSymbol
						ifTrue: [ self target ]
						ifFalse: [ self target name asSymbol ].
						
	currentValue := self targetSelector
						ifNil: [ (Smalltalk at: targetSymbol) perform: self getSelector ]
						ifNotNil: [:s | ((Smalltalk at: targetSymbol) perform: s) perform: self getSelector].
	
	^ (self hasDefault not or: [ (self default = currentValue) not ])
		ifTrue: [ 
			"The setting has changed, or there is no default value so we export it"
			"I assume here that the receiver is always a class"
			
			StartupAction 
				name: self label 
				code: (String streamContents: [:s |
							self targetSelector ifNotNil: [ s << '(' ].
							s << '(Smalltalk at: '.
							targetSymbol asSymbol printOn: s.
							self targetSelector ifNotNil: [:ts | s << ') perform: '. ts printOn: s ].
							s << ') perform: '.
							self setSelector asSymbol printOn: s.
							s << ' with: '.
							currentValue storeOn: s ])
				runOnce: true ]
		ifFalse: [ "We do nothing" nil ]! !
StartupLoader initialize!
